/*
 *
 * Copyright (C) 2016 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
 * Copyright (C) 2015 Sebastian Kügler <sebas@kde.org>
 *
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that the above copyright notice appear in
 * all copies and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the name of
 * the copyright holders not be used in advertising or publicity
 * pertaining to distribution of the software without specific,
 * written prior permission.  The copyright holders make no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied
 * warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 */

#ifndef GREENISLAND_OUTPUTMANAGEMENT_CLIENT_PROTOCOL_H
#define GREENISLAND_OUTPUTMANAGEMENT_CLIENT_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-client.h"

struct wl_client;
struct wl_resource;

struct greenisland_outputconfiguration;
struct greenisland_outputmanagement;
struct wl_output;

extern const struct wl_interface greenisland_outputmanagement_interface;
extern const struct wl_interface greenisland_outputconfiguration_interface;

#define GREENISLAND_OUTPUTMANAGEMENT_CREATE_CONFIGURATION	0

#define GREENISLAND_OUTPUTMANAGEMENT_CREATE_CONFIGURATION_SINCE_VERSION	1

static inline void
greenisland_outputmanagement_set_user_data(struct greenisland_outputmanagement *greenisland_outputmanagement, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) greenisland_outputmanagement, user_data);
}

static inline void *
greenisland_outputmanagement_get_user_data(struct greenisland_outputmanagement *greenisland_outputmanagement)
{
	return wl_proxy_get_user_data((struct wl_proxy *) greenisland_outputmanagement);
}

static inline uint32_t
greenisland_outputmanagement_get_version(struct greenisland_outputmanagement *greenisland_outputmanagement)
{
	return wl_proxy_get_version((struct wl_proxy *) greenisland_outputmanagement);
}

static inline void
greenisland_outputmanagement_destroy(struct greenisland_outputmanagement *greenisland_outputmanagement)
{
	wl_proxy_destroy((struct wl_proxy *) greenisland_outputmanagement);
}

static inline struct greenisland_outputconfiguration *
greenisland_outputmanagement_create_configuration(struct greenisland_outputmanagement *greenisland_outputmanagement)
{
	struct wl_proxy *id;

	id = wl_proxy_marshal_constructor((struct wl_proxy *) greenisland_outputmanagement,
			 GREENISLAND_OUTPUTMANAGEMENT_CREATE_CONFIGURATION, &greenisland_outputconfiguration_interface, NULL);

	return (struct greenisland_outputconfiguration *) id;
}

/**
 * greenisland_outputconfiguration - output configuration
 * @applied: configuration changes have been applied
 * @failed: configuration changes failed to apply
 *
 * This is a set of configuration changes for one or more outputs.
 *
 * Clients call requests on this resource in order to enable, set as
 * primary, change mode, change transform, move and change scaling factor
 * of one or more outputs. Once all attributes are configured, the client
 * asks the compositor to apply all changesets calling the apply request.
 * At this point the compositor signals back whether all changesets applied
 * successfully or failed to apply.
 */
struct greenisland_outputconfiguration_listener {
	/**
	 * applied - configuration changes have been applied
	 *
	 * Fired after the compositor has applied the changes
	 * successfully.
	 */
	void (*applied)(void *data,
			struct greenisland_outputconfiguration *greenisland_outputconfiguration);
	/**
	 * failed - configuration changes failed to apply
	 *
	 * Fired if the compositor rejects or failed to apply the
	 * changes.
	 */
	void (*failed)(void *data,
		       struct greenisland_outputconfiguration *greenisland_outputconfiguration);
};

static inline int
greenisland_outputconfiguration_add_listener(struct greenisland_outputconfiguration *greenisland_outputconfiguration,
					     const struct greenisland_outputconfiguration_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) greenisland_outputconfiguration,
				     (void (**)(void)) listener, data);
}

#define GREENISLAND_OUTPUTCONFIGURATION_ENABLE	0
#define GREENISLAND_OUTPUTCONFIGURATION_PRIMARY	1
#define GREENISLAND_OUTPUTCONFIGURATION_MODE	2
#define GREENISLAND_OUTPUTCONFIGURATION_TRANSFORM	3
#define GREENISLAND_OUTPUTCONFIGURATION_POSITION	4
#define GREENISLAND_OUTPUTCONFIGURATION_SCALE	5
#define GREENISLAND_OUTPUTCONFIGURATION_APPLY	6

#define GREENISLAND_OUTPUTCONFIGURATION_ENABLE_SINCE_VERSION	1
#define GREENISLAND_OUTPUTCONFIGURATION_PRIMARY_SINCE_VERSION	1
#define GREENISLAND_OUTPUTCONFIGURATION_MODE_SINCE_VERSION	1
#define GREENISLAND_OUTPUTCONFIGURATION_TRANSFORM_SINCE_VERSION	1
#define GREENISLAND_OUTPUTCONFIGURATION_POSITION_SINCE_VERSION	1
#define GREENISLAND_OUTPUTCONFIGURATION_SCALE_SINCE_VERSION	1
#define GREENISLAND_OUTPUTCONFIGURATION_APPLY_SINCE_VERSION	1

static inline void
greenisland_outputconfiguration_set_user_data(struct greenisland_outputconfiguration *greenisland_outputconfiguration, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) greenisland_outputconfiguration, user_data);
}

static inline void *
greenisland_outputconfiguration_get_user_data(struct greenisland_outputconfiguration *greenisland_outputconfiguration)
{
	return wl_proxy_get_user_data((struct wl_proxy *) greenisland_outputconfiguration);
}

static inline uint32_t
greenisland_outputconfiguration_get_version(struct greenisland_outputconfiguration *greenisland_outputconfiguration)
{
	return wl_proxy_get_version((struct wl_proxy *) greenisland_outputconfiguration);
}

static inline void
greenisland_outputconfiguration_destroy(struct greenisland_outputconfiguration *greenisland_outputconfiguration)
{
	wl_proxy_destroy((struct wl_proxy *) greenisland_outputconfiguration);
}

static inline void
greenisland_outputconfiguration_enable(struct greenisland_outputconfiguration *greenisland_outputconfiguration, struct wl_output *output, int32_t enable)
{
	wl_proxy_marshal((struct wl_proxy *) greenisland_outputconfiguration,
			 GREENISLAND_OUTPUTCONFIGURATION_ENABLE, output, enable);
}

static inline void
greenisland_outputconfiguration_primary(struct greenisland_outputconfiguration *greenisland_outputconfiguration, struct wl_output *output, int32_t primary)
{
	wl_proxy_marshal((struct wl_proxy *) greenisland_outputconfiguration,
			 GREENISLAND_OUTPUTCONFIGURATION_PRIMARY, output, primary);
}

static inline void
greenisland_outputconfiguration_mode(struct greenisland_outputconfiguration *greenisland_outputconfiguration, struct wl_output *output, int32_t mode_id)
{
	wl_proxy_marshal((struct wl_proxy *) greenisland_outputconfiguration,
			 GREENISLAND_OUTPUTCONFIGURATION_MODE, output, mode_id);
}

static inline void
greenisland_outputconfiguration_transform(struct greenisland_outputconfiguration *greenisland_outputconfiguration, struct wl_output *output, int32_t transform)
{
	wl_proxy_marshal((struct wl_proxy *) greenisland_outputconfiguration,
			 GREENISLAND_OUTPUTCONFIGURATION_TRANSFORM, output, transform);
}

static inline void
greenisland_outputconfiguration_position(struct greenisland_outputconfiguration *greenisland_outputconfiguration, struct wl_output *output, int32_t x, int32_t y)
{
	wl_proxy_marshal((struct wl_proxy *) greenisland_outputconfiguration,
			 GREENISLAND_OUTPUTCONFIGURATION_POSITION, output, x, y);
}

static inline void
greenisland_outputconfiguration_scale(struct greenisland_outputconfiguration *greenisland_outputconfiguration, struct wl_output *output, int32_t scale)
{
	wl_proxy_marshal((struct wl_proxy *) greenisland_outputconfiguration,
			 GREENISLAND_OUTPUTCONFIGURATION_SCALE, output, scale);
}

static inline void
greenisland_outputconfiguration_apply(struct greenisland_outputconfiguration *greenisland_outputconfiguration)
{
	wl_proxy_marshal((struct wl_proxy *) greenisland_outputconfiguration,
			 GREENISLAND_OUTPUTCONFIGURATION_APPLY);
}

#ifdef  __cplusplus
}
#endif

#endif
