/*
 *
 * Copyright (C) 2016 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
 * Copyright (C) 2015 Sebastian Kügler <sebas@kde.org>
 *
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that the above copyright notice appear in
 * all copies and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the name of
 * the copyright holders not be used in advertising or publicity
 * pertaining to distribution of the software without specific,
 * written prior permission.  The copyright holders make no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied
 * warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 */

#ifndef GREENISLAND_OUTPUTMANAGEMENT_SERVER_PROTOCOL_H
#define GREENISLAND_OUTPUTMANAGEMENT_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct greenisland_outputconfiguration;
struct greenisland_outputmanagement;
struct wl_output;

extern const struct wl_interface greenisland_outputmanagement_interface;
extern const struct wl_interface greenisland_outputconfiguration_interface;

/**
 * greenisland_outputmanagement - creates an output configuration to be
 *	applied later
 * @create_configuration: create a new output configuration
 *
 * The greenisland_outputmanagement global object allows clients to
 * create a configuration for one or more outputs to be applied later.
 */
struct greenisland_outputmanagement_interface {
	/**
	 * create_configuration - create a new output configuration
	 * @id: (none)
	 *
	 * Request an output configuration, which can be changed and then
	 * applied.
	 */
	void (*create_configuration)(struct wl_client *client,
				     struct wl_resource *resource,
				     uint32_t id);
};


/**
 * greenisland_outputconfiguration - output configuration
 * @enable: enable or disable an output
 * @primary: set an output as primary or secondary
 * @mode: change output mode
 * @transform: Transform outputdevice
 * @position: Position output relative to global
 * @scale: Set the scaling factor for this outputdevice
 * @apply: apply the changes to the outputs
 *
 * This is a set of configuration changes for one or more outputs.
 *
 * Clients call requests on this resource in order to enable, set as
 * primary, change mode, change transform, move and change scaling factor
 * of one or more outputs. Once all attributes are configured, the client
 * asks the compositor to apply all changesets calling the apply request.
 * At this point the compositor signals back whether all changesets applied
 * successfully or failed to apply.
 */
struct greenisland_outputconfiguration_interface {
	/**
	 * enable - enable or disable an output
	 * @output: (none)
	 * @enable: request to enable or disable this output
	 *
	 * Mark the output as enabled or disabled.
	 */
	void (*enable)(struct wl_client *client,
		       struct wl_resource *resource,
		       struct wl_resource *output,
		       int32_t enable);
	/**
	 * primary - set an output as primary or secondary
	 * @output: (none)
	 * @primary: request set this output as primary or secondary
	 *
	 * Mark the output as primary or secondary. Shell panels and
	 * user-interface elements are shown on primary outputs.
	 */
	void (*primary)(struct wl_client *client,
			struct wl_resource *resource,
			struct wl_resource *output,
			int32_t primary);
	/**
	 * mode - change output mode
	 * @output: output this mode change applies to
	 * @mode_id: aspired mode's id
	 *
	 * Sets the mode for a given output by its mode size (width and
	 * height) and refresh rate.
	 */
	void (*mode)(struct wl_client *client,
		     struct wl_resource *resource,
		     struct wl_resource *output,
		     int32_t mode_id);
	/**
	 * transform - Transform outputdevice
	 * @output: output this transformation change applies to
	 * @transform: transform enum
	 *
	 * Sets the transformation for a given output.
	 */
	void (*transform)(struct wl_client *client,
			  struct wl_resource *resource,
			  struct wl_resource *output,
			  int32_t transform);
	/**
	 * position - Position output relative to global
	 * @output: output this position applies to
	 * @x: position on the x-axis
	 * @y: position on the y-axis
	 *
	 * Sets the position for this output device in the global space.
	 * position describes the top-left border of the output in the
	 * global space. The left-most output's top-left border is 0, 0.
	 * There may be no gaps between outputs, the outputs have to be
	 * positioned stacked horizontally, vertically, or both.
	 */
	void (*position)(struct wl_client *client,
			 struct wl_resource *resource,
			 struct wl_resource *output,
			 int32_t x,
			 int32_t y);
	/**
	 * scale - Set the scaling factor for this outputdevice
	 * @output: output this mode change applies to
	 * @scale: scaling factor
	 *
	 * Sets the scaling factor for this outputdevice.
	 */
	void (*scale)(struct wl_client *client,
		      struct wl_resource *resource,
		      struct wl_resource *output,
		      int32_t scale);
	/**
	 * apply - apply the changes to the outputs
	 *
	 * Asks the compositor to apply changes to the outputs on the
	 * server side.
	 */
	void (*apply)(struct wl_client *client,
		      struct wl_resource *resource);
};

#define GREENISLAND_OUTPUTCONFIGURATION_APPLIED	0
#define GREENISLAND_OUTPUTCONFIGURATION_FAILED	1

#define GREENISLAND_OUTPUTCONFIGURATION_APPLIED_SINCE_VERSION	1
#define GREENISLAND_OUTPUTCONFIGURATION_FAILED_SINCE_VERSION	1

static inline void
greenisland_outputconfiguration_send_applied(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, GREENISLAND_OUTPUTCONFIGURATION_APPLIED);
}

static inline void
greenisland_outputconfiguration_send_failed(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, GREENISLAND_OUTPUTCONFIGURATION_FAILED);
}

#ifdef  __cplusplus
}
#endif

#endif
